"""Memory-related models.

- :class:`cml.models.responses.MemoryItem` — single memory from retrieval.
- :class:`cml.models.responses.ReadResponse`, :class:`cml.models.responses.WriteResponse`, etc.
- Request types in :mod:`cml.models.requests`.
- Enums (MemoryType, MemoryStatus, etc.) in :mod:`cml.models.enums`.

MemoryRecord and MemoryPacket may be added in a future release for
embedded-mode and server response parity.
"""
